@@ -27,6 +27,7 @@ from mautrix.appservice import AppService, IntentAPI
 from mautrix.bridge import BasePortal, NotificationDisabler, async_getter_lock
 from mautrix.errors import MatrixError, MForbidden
 from mautrix.types import (
+    BeeperMessageStatusEventContent,
     ContentURI,
     EncryptedFile,
     EventID,
@@ -34,7 +35,11 @@ from mautrix.types import (
     ImageInfo,
     MediaMessageEventContent,
     MessageEventContent,
+    MessageStatus,
+    MessageStatusReason,
     MessageType,
+    RelatesTo,
+    RelationType,
     RoomID,
     TextMessageEventContent,
     ThumbnailInfo,
@@ -188,13 +193,85 @@ class Portal(DBPortal, BasePortal):
     # endregion
     # region Matrix event handling
 
-    async def _send_error_notice(self, body: str, err: Exception):
-        await self._send_message(
-            self.main_intent,
-            TextMessageEventContent(msgtype=MessageType.NOTICE, body=f"\u26a0 {body}: {err}"),
+    async def _send_bridge_success(
+        self,
+        sender: u.User,
+        event_id: EventID,
+        event_type: EventType,
+        msgtype: MessageType | None = None,
+    ) -> None:
+        sender.send_remote_checkpoint(
+            status=MessageSendCheckpointStatus.SUCCESS,
+            event_id=event_id,
+            room_id=self.mxid,
+            event_type=event_type,
+            message_type=msgtype,
+        )
+        asyncio.create_task(self._send_message_status(event_id, err=None))
+        await self._send_delivery_receipt(event_id)
+
+    async def _send_bridge_error(
+        self,
+        sender: u.User,
+        err: Exception,
+        event_id: EventID,
+        event_type: EventType,
+        message_type: MessageType | None = None,
+    ) -> None:
+        sender.send_remote_checkpoint(
+            self._status_from_exception(err),
+            event_id,
+            self.mxid,
+            event_type,
+            message_type=message_type,
+            error=err,
         )
 
-    def _status_from_exception(self, e: Exception) -> MessageSendCheckpointStatus:
+        send_notice = not isinstance(err, NotImplementedError)
+        if self.config["bridge.delivery_error_reports"] and send_notice:
+            event_type_str = {
+                EventType.REACTION: "reaction",
+                EventType.ROOM_REDACTION: "redaction",
+            }.get(event_type, "message")
+            await self._send_message(
+                self.main_intent,
+                TextMessageEventContent(
+                    msgtype=MessageType.NOTICE,
+                    body=f"\u26a0 Your {event_type_str} may not have been bridged: {str(err)}",
+                ),
+            )
+        asyncio.create_task(self._send_message_status(event_id, err))
+
+    async def _send_message_status(self, event_id: EventID, err: Exception | None) -> None:
+        if not self.config["bridge.message_status_events"]:
+            return
+        intent = self.az.intent if self.encrypted else self.main_intent
+        status = BeeperMessageStatusEventContent(
+            network=self.bridge_info_state_key,
+            relates_to=RelatesTo(
+                rel_type=RelationType.REFERENCE,
+                event_id=event_id,
+            ),
+        )
+        if err:
+            status.status = MessageStatus.RETRIABLE
+            status.reason = MessageStatusReason.GENERIC_ERROR
+            status.error = str(err)
+            if isinstance(err, NotImplementedError):
+                status.status = MessageStatus.FAIL
+                status.reason = MessageStatusReason.UNSUPPORTED
+        else:
+            status.status = MessageStatus.SUCCESS
+        status.fill_legacy_booleans()
+
+        await intent.send_message_event(
+            room_id=self.mxid,
+            event_type=EventType.BEEPER_MESSAGE_STATUS,
+            content=status,
+        )
+
+    @staticmethod
+    def _status_from_exception(e: Exception) -> MessageSendCheckpointStatus:
         if isinstance(e, NotImplementedError):
             return MessageSendCheckpointStatus.UNSUPPORTED
         return MessageSendCheckpointStatus.PERM_FAILURE
@@ -202,32 +279,30 @@ class Portal(DBPortal, BasePortal):
     async def handle_matrix_message(
         self, sender: u.User, message: MessageEventContent, event_id: EventID
     ) -> None:
-        if not sender.client:
-            self.log.debug(f"Ignoring message {event_id} as user is not connected")
-            return
         try:
             await self._handle_matrix_message(sender, message, event_id)
         except Exception as e:
-            status = self._status_from_exception(e)
-            sender.send_remote_checkpoint(
-                status,
-                event_id,
-                self.mxid,
-                EventType.ROOM_MESSAGE,
-                message.msgtype,
-                error=e,
+            self.log.exception(f"Error handling Matrix event {event_id}")
+            await self._send_bridge_error(
+                sender, e, event_id, EventType.ROOM_MESSAGE, message.msgtype
+            )
+        else:
+            await self._send_bridge_success(
+                sender, event_id, EventType.ROOM_MESSAGE, message.msgtype
             )
-            await self._send_error_notice("Your message may not have been bridged", e)
 
     async def _handle_matrix_message(
         self, sender: u.User, message: MessageEventContent, event_id: EventID
     ) -> None:
+        if not sender.client:
+            raise NotImplementedError("user is not connected")
         request_id = str(sender.client.new_request_id())
         self._reqid_dedup.add(request_id)
-        text = message.body
         media_id = None
-        if message.msgtype == MessageType.EMOTE:
-            text = f"/me {text}"
+        if message.msgtype == MessageType.TEXT:
+            text = message.body
+        elif message.msgtype == MessageType.EMOTE:
+            text = f"/me {message.body}"
         elif message.msgtype.is_media:
             if message.file and decrypt_attachment:
                 data = await self.main_intent.download_media(message.file.url)
@@ -243,17 +318,11 @@ class Portal(DBPortal, BasePortal):
             upload_resp = await sender.client.upload(data, mime_type=mime_type)
             media_id = upload_resp.media_id
             text = ""
+        else:
+            raise NotImplementedError(f"unsupported msgtype '{message.msgtype.value}'")
         resp = await sender.client.conversation(self.twid).send(
             text, media_id=media_id, request_id=request_id
         )
-        await self._send_delivery_receipt(event_id)
-        sender.send_remote_checkpoint(
-            status=MessageSendCheckpointStatus.SUCCESS,
-            event_id=event_id,
-            room_id=self.mxid,
-            event_type=EventType.ROOM_MESSAGE,
-            message_type=message.msgtype,
-        )
         resp_msg_id = int(resp.entries[0].message.id)
         self._msgid_dedup.appendleft(resp_msg_id)
         msg = DBMessage(mxid=event_id, mx_room=self.mxid, twid=resp_msg_id, receiver=self.receiver)
@@ -267,10 +336,10 @@ class Portal(DBPortal, BasePortal):
         try:
             await self._handle_matrix_reaction(sender, event_id, reacting_to, reaction_val)
         except Exception as e:
-            status = self._status_from_exception(e)
             self.log.exception(f"Failed to react to {event_id}")
-            sender.send_remote_checkpoint(status, event_id, self.mxid, EventType.REACTION, error=e)
-            await self._send_error_notice(f"Failed to react to {event_id}", e)
+            await self._send_bridge_error(sender, e, event_id, EventType.REACTION)
+        else:
+            await self._send_bridge_success(sender, event_id, EventType.REACTION)
 
     async def _handle_matrix_reaction(
         self, sender: u.User, event_id: EventID, reacting_to: EventID, reaction_val: str
@@ -278,24 +347,19 @@ class Portal(DBPortal, BasePortal):
         reaction = ReactionKey.from_emoji(reaction_val)
         message = await DBMessage.get_by_mxid(reacting_to, self.mxid)
         if not message:
-            raise ValueError(f"Ignoring reaction to unknown event {reacting_to}")
+            raise NotImplementedError(f"Unknown reaction target event")
 
         async with self._reaction_lock:
             dedup_id = (message.twid, sender.twid, reaction)
             self._reaction_dedup.appendleft(dedup_id)
 
-            existing = await DBReaction.get_by_twid(message.twid, message.receiver, sender.twid)
+            existing = await DBReaction.get_by_message_twid(
+                message.twid, message.receiver, sender.twid
+            )
             if existing and existing.reaction == reaction:
                 return
 
             await sender.client.conversation(self.twid).react(message.twid, reaction)
-            sender.send_remote_checkpoint(
-                MessageSendCheckpointStatus.SUCCESS,
-                event_id,
-                self.mxid,
-                EventType.REACTION,
-            )
-            await self._send_delivery_receipt(event_id)
             await self._upsert_reaction(
                 existing, self.main_intent, event_id, message, sender, reaction
             )
@@ -307,47 +371,30 @@ class Portal(DBPortal, BasePortal):
         try:
             await self._handle_matrix_redaction(sender, event_id)
         except Exception as e:
-            status = self._status_from_exception(e)
-            sender.send_remote_checkpoint(
-                status, event_id, self.mxid, EventType.ROOM_REDACTION, error=e
-            )
-            await self._send_error_notice(f"Failed to redact {event_id}", e)
+            self.log.exception(f"Error handling redaction of {event_id}")
+            await self._send_bridge_error(sender, e, redaction_event_id, EventType.ROOM_REDACTION)
         else:
-            sender.send_remote_checkpoint(
-                MessageSendCheckpointStatus.SUCCESS,
-                redaction_event_id,
-                self.mxid,
-                EventType.ROOM_REDACTION,
-            )
-            await self._send_delivery_receipt(redaction_event_id)
+            await self._send_bridge_success(sender, redaction_event_id, EventType.ROOM_REDACTION)
 
     async def _handle_matrix_redaction(self, sender: u.User, event_id: EventID) -> None:
-        assert self.mxid, "MXID is None"
-
         async with self._reaction_lock:
             reaction = await DBReaction.get_by_mxid(event_id, self.mxid)
             if reaction:
-                try:
-                    await reaction.delete()
-                    await sender.client.conversation(self.twid).delete_reaction(
-                        reaction.tw_msgid, reaction.reaction
-                    )
-                except Exception:
-                    self.log.exception("Removing reaction failed")
-                    raise
-                else:
-                    self.log.trace(f"Removed {reaction} after Matrix redaction")
-
+                await reaction.delete()
+                await sender.client.conversation(self.twid).delete_reaction(
+                    reaction.tw_msgid, reaction.reaction
+                )
+                self.log.trace(f"Removed {reaction} after Matrix redaction")
                 return
 
-        raise NotImplementedError("Message redactions are not supported.")
+        raise NotImplementedError("Message redactions are not supported")
 
     async def handle_matrix_leave(self, user: u.User) -> None:
         if self.is_direct:
             self.log.info(f"{user.mxid} left private chat portal with {self.twid}")
             if user.twid == self.receiver:
                 self.log.info(
-                    f"{user.mxid} was the recipient of this portal. " "Cleaning up and deleting..."
+                    f"{user.mxid} was the recipient of this portal. Cleaning up and deleting..."
                 )
                 await self.cleanup_and_delete()
         else:
@@ -531,15 +578,22 @@ class Portal(DBPortal, BasePortal):
             file=reuploaded_info.decryption_info,
             external_url=media.media_url_https,
         )
-        if message.attachment.video:
+        if message.attachment.video or (
+            message.attachment.animated_gif and reuploaded_info.mime_type.startswith("video/")
+        ):
             content.msgtype = MessageType.VIDEO
             content.info = VideoInfo(
                 mimetype=reuploaded_info.mime_type,
                 size=reuploaded_info.size,
                 width=media.original_info.width,
                 height=media.original_info.height,
-                duration=media.video_info.duration_millis // 1000,
+                duration=media.video_info.duration_millis or None,
             )
+            if message.attachment.animated_gif:
+                content.info["fi.mau.loop"] = True
+                content.info["fi.mau.autoplay"] = True
+                content.info["fi.mau.hide_controls"] = True
+                content.info["fi.mau.no_audio"] = True
         elif message.attachment.photo or message.attachment.animated_gif:
             content.msgtype = MessageType.IMAGE
             content.info = ImageInfo(
@@ -604,8 +658,10 @@ class Portal(DBPortal, BasePortal):
                 return
             self._reaction_dedup.appendleft(dedup_id)
 
-        existing = await DBReaction.get_by_twid(msg_id, self.receiver, sender.twid)
+        existing = await DBReaction.get_by_message_twid(msg_id, self.receiver, sender.twid)
         if existing and existing.reaction == reaction:
+            if not existing.tw_reaction_id:
+                await existing.update_id(reaction_id)
             return
 
         message = await DBMessage.get_by_twid(msg_id, self.receiver)
@@ -621,7 +677,7 @@ class Portal(DBPortal, BasePortal):
     async def handle_twitter_reaction_remove(
         self, sender: p.Puppet, msg_id: int, key: ReactionKey
     ) -> None:
-        reaction = await DBReaction.get_by_twid(msg_id, self.receiver, sender.twid)
+        reaction = await DBReaction.get_by_message_twid(msg_id, self.receiver, sender.twid)
         if reaction and reaction.reaction == key:
             try:
                 await sender.intent_for(self).redact(reaction.mx_room, reaction.mxid)
@@ -635,11 +691,13 @@ class Portal(DBPortal, BasePortal):
     ) -> None:
         message = await DBMessage.get_by_twid(read_up_to, self.receiver)
         if not message:
-            self.log.debug(
-                f"Ignoring read receipt from {sender.twid} "
-                f"up to unknown message {read_up_to} ({historical=})"
-            )
-            return
+            message = await DBReaction.get_by_reaction_twid(read_up_to, self.receiver)
+            if not message:
+                self.log.debug(
+                    f"Ignoring read receipt from {sender.twid} "
+                    f"up to unknown message {read_up_to} ({historical=})"
+                )
+                return
 
         self.log.debug(
             f"{sender.twid} read messages up to {read_up_to} ({message.mxid}, {historical=})"
@@ -755,21 +813,25 @@ class Portal(DBPortal, BasePortal):
 
     async def _fetch_backfill_entries(
         self, source: u.User, limit: int
-    ) -> list[MessageEntry | ReactionCreateEntry | ReactionDeleteEntry]:
+    ) -> list[MessageEntry | ReactionCreateEntry]:
         conv = source.client.conversation(self.twid)
-        entries = []
+        entries: list[MessageEntry | ReactionCreateEntry] = []
+        message_count = 0
         self.log.debug("Fetching up to %d messages through %s", limit, source.twid)
         try:
             max_id = None
             while True:
-                resp = await conv.fetch(max_id=max_id, include_info=False)
+                resp = await conv.fetch(max_id=max_id)
                 max_id = resp.min_entry_id
                 for entry in resp.entries:
-                    if entry:
-                        entries += entry.all_types
-                    if len(entries) >= limit:
+                    if entry and entry.message:
+                        entries.append(entry.message)
+                        message_count += 1
+                        if entry.message.message_reactions:
+                            entries += entry.message.message_reactions
+                    if message_count >= limit:
                         break
-                if len(entries) >= limit:
+                if message_count >= limit:
                     self.log.debug("Got more messages than limit")
                     break
                 elif resp.status == TimelineStatus.AT_END:
@@ -777,12 +839,10 @@ class Portal(DBPortal, BasePortal):
                     break
         except Exception:
             self.log.warning("Exception while fetching messages", exc_info=True)
-
-        return [
-            entry
-            for entry in entries
-            if isinstance(entry, (MessageEntry, ReactionCreateEntry, ReactionDeleteEntry))
-        ]
+        if message_count != len(entries):
+            # Needs sorting to get reactions into correct place
+            entries.sort(key=lambda ent: (ent.time, ent.id), reverse=True)
+        return entries
 
     async def _invite_own_puppet_backfill(self, source: u.User) -> set[IntentAPI]:
         backfill_leave = set()
@@ -796,18 +856,14 @@ class Portal(DBPortal, BasePortal):
         return backfill_leave
 
     async def _handle_backfill_entry(
-        self, source: u.User, entry: MessageEntry | ReactionCreateEntry | ReactionDeleteEntry
+        self, source: u.User, entry: MessageEntry | ReactionCreateEntry
     ) -> None:
         sender = await p.Puppet.get_by_twid(int(entry.sender_id))
         if isinstance(entry, MessageEntry):
             await self.handle_twitter_message(source, sender, entry.message_data, entry.request_id)
         if isinstance(entry, ReactionCreateEntry):
             await self.handle_twitter_reaction_add(
-                sender, int(entry.message_id), entry.reaction_key, entry.time
-            )
-        elif isinstance(entry, ReactionDeleteEntry):
-            await self.handle_twitter_reaction_remove(
-                sender, int(entry.message_id), entry.reaction_key
+                sender, int(entry.message_id), entry.reaction_key, entry.time, int(entry.id)
             )
 
     # endregion
@@ -916,7 +972,7 @@ class Portal(DBPortal, BasePortal):
             initial_state.append(
                 {
                     "type": "m.room.encryption",
-                    "content": {"algorithm": "m.megolm.v1.aes-sha2"},
+                    "content": self.get_encryption_state_event_json(),
                 }
             )
             if self.is_direct:
